#!/usr/bin/env bash
#############################################
# USER INPUT

# Set the time interval for visbility computations
DATE_START="2018-09-01 00:00"
DATE_STOP="2018-09-02 00:00"
#DATE_STOP="2018-06-30 23:59"

# Short time interval for tests
#DATE_START="2017-02-01 03:00"
#DATE_STOP="2017-02-01 03:59"

#
#DATE_START="2018-01-01 02:00"
#DATE_STOP="2018-01-01 02:59"
#DATE_START="2018-04-16 10:00"
#DATE_STOP="2018-04-17 08:59"


# Remember that Solar ephemeris in sun_from_horizons.dat are only until 2021-10-21
# The script will not be able to work after this date unless you update sun_from_horizons.dat

# Automatically get the latest orbit
RA_ORBIT_FILE="automatic"
# or you may specify the orbit file manually, for example:
#RA_ORBIT_FILE="/home/kirx/fakerat/RA170716-200411_v01.org"
### Number of fakerat threads running in parallell
#NUMBER_OF_THREADS=6
#
# Change to 'no' to disable YG and HH
#EXTRA_TRACKING_STATIONS=yes
EXTRA_TRACKING_STATIONS=no
#
# 0 - no debug output
DEBUGLEVEL=0
#
#############################################
### You probably do not want to change anything below this line
#############################################
###
FAKERAT_PATH=`readlink -f $0`
FAKERAT_PATH=`dirname $FAKERAT_PATH`
#FAKERATLIB_PATH=`dirname $FAKERAT_PATH`/fakeratlib-4.9.4_64__v23
FAKERATLIB_PATH=`dirname $FAKERAT_PATH`/fakerat-5.4.0_64.v24.beta
#############################################


### Check the FakeRaT installation consistency 
if [ ! -d "$FAKERAT_PATH" ];then
 echo "ERROR: cannot find the FakeRaT directory $FAKERAT_PATH" >> /dev/stderr
 exit 1
fi
if [ ! -d "$FAKERATLIB_PATH" ];then
 echo "ERROR: cannot find the FakeRaT library directory $FAKERATLIB_PATH" >> /dev/stderr
 exit 1
fi
for FILETOTEST in astro  fakerat  fakerat_menu  fakeratlib.a  fakeratm  orblib.a  vlblib.a ;do
 if [ ! -f "$FAKERATLIB_PATH/$FILETOTEST" ];then
  echo "ERROR: cannot find file $FAKERATLIB_PATH/$FILETOTEST" >> /dev/stderr
  exit 1
 fi
done
for FILETOTEST in autosched.sh stations.dat stations.cat ;do
 if [ ! -f "$FAKERAT_PATH/$FILETOTEST" ];then
  echo "ERROR: cannot find file $FAKERAT_PATH/$FILETOTEST" >> /dev/stderr
  exit 1
 fi
done

if [ -z "$DATE_START" ];then
 echo "ERROR: DATE_START is not set" >> /dev/stderr
 exit 1
fi
if [ -z "$DATE_STOP" ];then
 echo "ERROR: DATE_STOP is not set" >> /dev/stderr
 exit 1
fi
if [ "$DATE_START" = "$DATE_STOP" ];then
 echo "ERROR: DATE_START = DATE_STOP" >> /dev/stderr
 exit 1
fi



############### The following will test if the needed external programs are installed ###############
SCRIPTNAME=`basename $0`
MISSING_PROGRAM=0
LIST_OF_MISSING_PROGRAMS=""

#################### Check SCHED ####################
# Add one likely place where SCHED might be installed
if [ -d $HOME/sched/bin ];then
 export PATH=$PATH:$HOME/sched/bin/
fi
# Find the SCHED executable
SCHEDEXECUTABLE=`command -v sched 2>/dev/null`
if [ -z $SCHEDEXECUTABLE ];then
 if [ -x $SCHED/bin/sched ];then
  SCHEDEXECUTABLE=$SCHED/bin/sched
 else
  # Yes it will not work, but the error message will look meaningful
  SCHEDEXECUTABLE="sched"
 fi
fi
# export it just in case... we are gonna use it from inside a forked function
export SCHEDEXECUTABLE
#####################################################

for TESTED_PROGRAM in gcc awk sed bc cat touch fold head tail grep skycoor readlink curlftpfs date $SCHEDEXECUTABLE ;do
 if ! command -v $TESTED_PROGRAM &>/dev/null ;then
  MISSING_PROGRAM=1
  LIST_OF_MISSING_PROGRAMS="$LIST_OF_MISSING_PROGRAMS  $TESTED_PROGRAM"
  echo -e "An external program $TESTED_PROGRAM - \033[01;31mNOT found\033[00m"
  if [ "$TESTED_PROGRAM" == "skycoor" ];then
   echo "skycoor program is part of WCSTools, see http://tdc-www.harvard.edu/wcstools/"
  fi
  if [ "$TESTED_PROGRAM" == "sched" ];then
   echo "NRAO SCHED may be found here: http://www.aoc.nrao.edu/software/sched/"
  fi
 fi
done
if [ $MISSING_PROGRAM -eq 1 ] ;then
 echo -e "
ERROR: some programs needed to run $SCRIPTNAME were not found in the \$PATH!
\033[01;31mPlease install the following programs:

$LIST_OF_MISSING_PROGRAMS\033[00m
"
 exit 1
fi

#####################################################################################################

function set_number_of_threads {
 # Uncomment to disable multithreading
 #echo "1"
 #return 0
 #
 
 local DEFAULT_NUMBER_OF_THREADS=4

 SYSTEM_TYPE=`uname`
 if [ "$SYSTEM_TYPE" = "Linux" ];then
  NUMBER_OF_THREADS=`cat /proc/cpuinfo | grep -c 'processor'`
 else
  NUMBER_OF_THREADS=`sysctl -a | grep "hw.ncpu" | awk '{print $2}'`
 fi

 # If $OMP_NUM_THREADS is set - stick with this value
 if [ ! -z "$OMP_NUM_THREADS" ];then
  NUMBER_OF_THREADS="$OMP_NUM_THREADS"
 fi
 
 # check that NUMBER_OF_THREADS is areasonable number
 if [ -z "$NUMBER_OF_THREADS" ];then
  NUMBER_OF_THREADS=$DEFAULT_NUMBER_OF_THREADS
 fi
 TEST=`echo "$NUMBER_OF_THREADS > 0" | bc -q 2>/dev/null`
 re='^[0-9]+$'
 if ! [[ $TEST =~ $re ]] ; then
  NUMBER_OF_THREADS=$DEFAULT_NUMBER_OF_THREADS
 elif [ $TEST -eq 0 ];then
  NUMBER_OF_THREADS=$DEFAULT_NUMBER_OF_THREADS
 fi
 
 echo "$NUMBER_OF_THREADS"
 
 return 0;
}


## This function will update the orbit filename in menurat_defaults
function replace_orbit_filename_in_menurat_defaults {
 # Check input
 if [ -z "$1" ];then
  echo "ERROR in replace_orbit_filename_in_menurat_defaults(): orbit file name is not specified" >> /dev/stderr
  return 1
 fi
 NEWORBITFILENAME="$1"
 if [ ${#NEWORBITFILENAME} -gt 30 ];then
  echo "ERROR in replace_orbit_filename_in_menurat_defaults(): NEWORBITFILENAME is too long! Should be between 16 and 30 symbols." >> /dev/stderr
  return 1
 fi
 if [ ${#NEWORBITFILENAME} -lt 16 ];then
  echo "ERROR in replace_orbit_filename_in_menurat_defaults(): NEWORBITFILENAME is too short! Should be between 16 and 30 symbols." >> /dev/stderr
  return 1
 fi
 if [ ! -f menurat_defaults ];then
  echo "ERROR in replace_orbit_filename_in_menurat_defaults(): menurat_defaults file is not found" >> /dev/stderr
  return 1
 fi
 #
 ORBITLINE0=`grep 'Orbit ' menurat_defaults`
 if [ $? -ne 0 ];then
  echo "ERROR in replace_orbit_filename_in_menurat_defaults(): cannot find line Orbit" >> /dev/stderr
  return 1
 fi
 ORBITLINE1=`grep 'Orbit1' menurat_defaults`
 if [ $? -ne 0 ];then
  echo "ERROR in replace_orbit_filename_in_menurat_defaults(): cannot find line Orbit1" >> /dev/stderr
  return 1
 fi
 
 ORBITLINE0_BEFORE=`echo "$ORBITLINE0" | awk '{print $1"|"$2}' FS='|'`
 ORBITLINE0_AFTER=`echo "$ORBITLINE0" | awk '{print $4"|"$5"|"$6"|"$7"|"}' FS='|'`
 ORBITLINE1_BEFORE=`echo "$ORBITLINE1" | awk '{print $1"|"$2}' FS='|'`
 ORBITLINE1_AFTER=`echo "$ORBITLINE1" | awk '{print $4"|"$5"|"$6"|"$7"|"}' FS='|'`
 
 NEWORBITFILENAME_LINE1=`echo $NEWORBITFILENAME | fold -s -w15 | head -n1`
 NEWORBITFILENAME_LINE2=`echo $NEWORBITFILENAME | fold -s -w15 | tail -n1`
 
 # Add white spaces to NEWORBITFILENAME_LINE2
 while [ ${#NEWORBITFILENAME_LINE2} -lt 15 ];do
  NEWORBITFILENAME_LINE2="$NEWORBITFILENAME_LINE2 "
 done
 
 REPLACE_ORBITLINE0=`echo "$ORBITLINE0_BEFORE|$NEWORBITFILENAME_LINE1|$ORBITLINE0_AFTER"`
 REPLACE_ORBITLINE1=`echo "$ORBITLINE1_BEFORE|$NEWORBITFILENAME_LINE2|$ORBITLINE1_AFTER"`
 
 # Do the change only if wee do need to change something
 MENURAT_UPDATE_NEEDED=0
 if [ "$ORBITLINE0" != "$REPLACE_ORBITLINE0" ];then
  MENURAT_UPDATE_NEEDED=1
 fi
 if [ "$ORBITLINE1" != "$REPLACE_ORBITLINE1" ];then
  MENURAT_UPDATE_NEEDED=1
 fi
 if [ $MENURAT_UPDATE_NEEDED -eq 1 ];then
  echo "Updating menurat_defaults" >> /dev/stderr
  cat menurat_defaults | sed "s:$ORBITLINE0:$REPLACE_ORBITLINE0:g" | sed "s:$ORBITLINE1:$REPLACE_ORBITLINE1:g" > menurat_defaults$$.tmp
  mv menurat_defaults$$.tmp menurat_defaults
  #
  cp -f menurat_defaults menurat_defaults.1
  #
  # More important thing - create file with the new orbit commands
  echo "ORBIT1     = $NEWORBITFILENAME_LINE1
ORBIT2     = $NEWORBITFILENAME_LINE2
" > .orbitlines_for_fakerat_command_files
 else
  echo "No need to update menurat_defaults" >> /dev/stderr
 fi

 return 0
}


## This function will compare two files by computing their MD5 checksums with md5sum
function check_if_two_files_are_the_same {
 FILE1="$1"
 FILE2="$2"
 command -v md5sum &>/dev/null
 if [ $? -ne 0 ];then
  echo "Cannot find the command: md5sum" >> /dev/stderr
  echo "Assuming files $FILE1 and $FILE2 are different" >> /dev/stderr
  return 0
 fi
 if [ ! -f "$FILE1" ];then
  return 0
 fi
 if [ ! -f "$FILE2" ];then
  return 0
 fi
 MD5SUM_OF_FILE1=`md5sum $FILE1 | awk '{print $1}'`
 MD5SUM_OF_FILE2=`md5sum $FILE2 | awk '{print $1}'`
 if [ "$MD5SUM_OF_FILE1" != "$MD5SUM_OF_FILE2" ];then
  echo "Files $FILE1 and $FILE2 are not identical" >> /dev/stderr
  return 0
 fi
 return 1
}

## This function will set the random session key in attempt to avoid 
## file collisions if other instances of the script are running at the same time.
function set_session_key {
 if [ -r /dev/urandom ];then
  local RANDOMFILE=/dev/urandom
 elif [ -r /dev/random ];then
  local RANDOMFILE=/dev/random
 else
  echo "ERROR: cannot find /dev/random" >> /dev/stderr
  local RANDOMFILE=""
 fi
 if [ "$RANDOMFILE" != "" ];then
  local SESSION_KEY="$$"_`tr -cd a-zA-Z0-9 < $RANDOMFILE | head -c 12`
 else
  local SESSION_KEY="$$"
 fi
 echo "$SESSION_KEY"
}



## This function will get the latest long-term orbit prediction to compute distance from a tracing station
function get_latest_orbit {
 # Check if curlftpfs is installed
 command -v curlftpfs &> /dev/null
 if [ $? -ne 0 ];then
  echo "
#####################################
 WARNING: curlftpfs is not installed  -- I can't automatically check for orbit updates!
 WARNING: the script will continue ASSUMING the currently installed orbit is suficiently fresh.
#####################################
" >> /dev/stderr
  # Print the old orbit file name
  if [ -f ra_orbit ];then
   readlink ra_orbit
  elif [ orbit/ra_orbit ];then
   echo orbit/`readlink orbit/ra_orbit`
  else
   echo "ERROR: cannot find a link to the old orbit file in orbit/ra_orbit" >> /dev/stderr
   exit 1
  fi
  return 1
 fi
 local FTPURL_ORBIT_DIR_ENDS_WITH_SLASH='ftp://radik:Zap069_qq@webinet.asc.rssi.ru/radioastron/oddata/predict/long/'
 # The global variable SESSION_KEY should be set externally
 if [ -z $SESSION_KEY ];then
  echo "ERROR: SESSION_KEY variable is not set!" >> /dev/stderr
  exit 1
 fi
 local CURLFTPFS_MOUNT_DIR_ORBIT="/tmp/autofake_sh_get_latest_orbit_$SESSION_KEY"
 # Create the directory if it does not exist
 if [ ! -d $CURLFTPFS_MOUNT_DIR_ORBIT/ ];then
  mkdir $CURLFTPFS_MOUNT_DIR_ORBIT/
 else
  fusermount -u $CURLFTPFS_MOUNT_DIR_ORBIT/
 fi
 # mount read-only
 curlftpfs -r "$FTPURL_ORBIT_DIR_ENDS_WITH_SLASH" $CURLFTPFS_MOUNT_DIR_ORBIT/
 if [ $? -ne 0 ];then
  echo "WARNING: cannot mount $FTPURL_ORBIT_DIR_ENDS_WITH_SLASH" >> /dev/stderr
 else
  # Check if there are orbit files in the remote directory
  for REMOTEORBITFILE in $CURLFTPFS_MOUNT_DIR_ORBIT/RA*_v2.org ;do
   if [ -f $REMOTEORBITFILE ];then
    echo "OK, found some orbit files on the FTP server" >> /dev/stderr
    local ORBITFILE=`ls -t $CURLFTPFS_MOUNT_DIR_ORBIT/RA*_v2.org | head -n1`
    local BASENAME_ORBITFILE=`basename $ORBITFILE`
    echo "The latest orbit file found on the FTP server is $BASENAME_ORBITFILE" >> /dev/stderr
    echo "We are in $PWD, checking if there are local orbit files..." >> /dev/stderr
    # Check if there are local orbit files
    for LOCALORBITFILE in RA*_v2.org ;do
     if [ -f $LOCALORBITFILE ];then
      local LATESTLOCALORBITFILE=`ls -t RA*_v2.org | head -n1`
      echo "The latest local orbit file is $LATESTLOCALORBITFILE" >> /dev/stderr
      # If there is a newer orbit file
      if [ "$LATESTLOCALORBITFILE" != "$BASENAME_ORBITFILE" ];then
       echo "The latest local file is not the same as the latest remote one: $LATESTLOCALORBITFILE (local),  $BASENAME_ORBITFILE (remote)" >> /dev/stderr
       # Copy the remore file cutting-off its header
       echo -e "\033[01;34mDownloading the new orbit file $BASENAME_ORBITFILE\033[00m
It \033[01;34mwill take time\033[00m as the orbit file is large. 
Please, \033[01;34mDO NOT INTERRUPT THIS PROCESS!\033[00m" >> /dev/stderr
       cp -v $ORBITFILE new_orbit$SESSION_KEY.org >> /dev/stderr
       if [ $? -eq 0 ];then
        # Check if the copy is identical to the original by computing checksums
        # Yes, sadly, for this we have to download the original again
        echo -e "Checking if the orbit file is downloaded correctly... 
This check \033[01;34mwill take time\033[00m as the orbit file is large. 
Please, \033[01;34mDO NOT INTERRUPT THIS PROCESS!\033[00m" >> /dev/stderr
        check_if_two_files_are_the_same $ORBITFILE new_orbit$SESSION_KEY.org
        if [ $? -eq 1 ];then 
         echo -e "\033[01;32mYes, the orbit file is downloaded correctly.\033[00m" >> /dev/stderr
         cat new_orbit$SESSION_KEY.org | tail -n +15 > "$BASENAME_ORBITFILE"
         # OK, now we have the newest orbit file locally
        else
         echo -e "\033[01;31mNO\033[00m, the downlaoded file did not pass the check! The corrupted download will be removed." >> /dev/stderr
        fi
       else
        echo "ERROR copying the new orbit file!" >> /dev/stderr
       fi # if cp -v $ORBITFILE new_orbit$SESSION_KEY.org is OK
       # Anyway, remove the downloaded file
       rm -f new_orbit$SESSION_KEY.org
       # end of remote file copy
      fi
     else
      # No local orbit files found!

      # Copy the remore file cutting-off its header
      echo -e "\033[01;34mDownloading the new orbit file $BASENAME_ORBITFILE\033[00m
It \033[01;34mwill take time\033[00m as the orbit file is large. 
Please, \033[01;34mDO NOT INTERRUPT THIS PROCESS!\033[00m" >> /dev/stderr
      cp -v $ORBITFILE new_orbit$SESSION_KEY.org >> /dev/stderr
      if [ $? -eq 0 ];then
       # Check if the copy is identical to the original by computing checksums
       # Yes, sadly, for this we have to download the original again
       echo -e "Checking if the orbit file is downloaded correctly... 
This check \033[01;34mwill take time\033[00m as the orbit file is large. 
Please, \033[01;34mDO NOT INTERRUPT THIS PROCESS!\033[00m" >> /dev/stderr
       check_if_two_files_are_the_same $ORBITFILE new_orbit$SESSION_KEY.org
       if [ $? -eq 1 ];then 
        echo -e "\033[01;32mYes, the orbit file is downloaded correctly.\033[00m" >> /dev/stderr
        cat new_orbit$SESSION_KEY.org | tail -n +15 > "$BASENAME_ORBITFILE"
        # OK, now we have the newest orbit file locally
       else
        echo -e "\033[01;31mNO\033[00m, the downlaoded file did not pass the check! The corrupted download will be removed." >> /dev/stderr
       fi
      else
       echo "ERROR copying the new orbit file!" >> /dev/stderr
      fi # if cp -v $ORBITFILE new_orbit$SESSION_KEY.org is OK
      # Anyway, remove the downloaded file
      rm -f new_orbit$SESSION_KEY.org
      # end of remote file copy
     fi
     break
    done # for LOCALORBITFILE in RA*.org ;do
   else
    echo "ERROR: no orbit files RA*.org could be found on the FTP server" >> /dev/stderr
    # Will try to use the local ones
   fi # if [ -f $REMOTEORBITFILE ];then
   break # anyway
  done # for REMOTEORBITFILE in $CURLFTPFS_MOUNT_DIR_ORBIT/RA*.org ;do
   
 fi # if curlftpfs mounted well

 ### Find the latest orbit prediction file, we may have just updated it
 # Check if there are local orbit files
 for LOCALORBITFILE in RA*_v2.org ;do
  if [ -f $LOCALORBITFILE ];then
   echo "Found orbit files locally" >> /dev/stderr
   local ORBITFILE=`ls -t RA*_v2.org | head -n1`
   echo "The latest orbit file is $ORBITFILE" >> /dev/stderr
   break
  else
   echo "ERROR: no orbit files RA*_v2.org could be found" >> /dev/stderr
   exit 1
  fi
 done

 ### Clean up 
 # Just for the paranoia sake, because we remove the directory recursively below
 if [ "$CURLFTPFS_MOUNT_DIR_ORBIT" == "" ];then
  echo "FATAL ERROR: corrupted variable CURLFTPFS_MOUNT_DIR_ORBIT" >> /dev/stderr
  exit 1
 fi
 if [ "$CURLFTPFS_MOUNT_DIR_ORBIT" == "/" ];then
  echo "FATAL ERROR: corrupted variable CURLFTPFS_MOUNT_DIR_ORBIT" >> /dev/stderr
  exit 1
 fi
 fusermount -u $CURLFTPFS_MOUNT_DIR_ORBIT/ && rm -rf $CURLFTPFS_MOUNT_DIR_ORBIT

 # Return the result
 echo "$ORBITFILE"
}

## This function will set the appropriate symlink to the orbit file for FakeRaT
function set_orbit_symlink {
 #
 # The new FakeRaT doesn't need the symlink but sets the orbit filename from the menu file menurat_defaults
 # The function replace_orbit_filename_in_menurat_defaults() below will take care of the update.
 # We will create the symlink just in case.
 #
 ### Get the right orbit ###
 STARTDIR=$PWD
 if [ ! -d orbit ];then
  mkdir orbit
 fi
 cd orbit
 if [ "$RA_ORBIT_FILE" = "automatic" ];then
  # automatically get the latest orbit file
  RA_ORBIT_FILE=$(get_latest_orbit)
  # Delete the oldest orbit file
  OLDEST_ORBIT_FILE=`ls -t RA*_v2.org | tail -n 1`
  # make sure the oldest orbit file is not the same as the new one
  if [ "$OLDEST_ORBIT_FILE" != "$RA_ORBIT_FILE" ];then
   rm -f "$OLDEST_ORBIT_FILE"
  else
   echo "Not deleting the old orbit file - it's the same as the new one!" >> /dev/stderr
  fi
 fi
 # 
 # Check if the orbit file is here
 if [ ! -s "$RA_ORBIT_FILE" ];then
  echo "ERROR: cannot find the predicted orbit file $RA_ORBIT_FILE" >> /dev/stderr
  exit 1
 else
  echo "Using the predicted orbit file $RA_ORBIT_FILE" >> /dev/stderr
 fi
 #### No need to remove orbit file header with the new FakeRaT ####
 ## Check if we need to chop-off its head
 head -n 10 $RA_ORBIT_FILE |grep "META_START" &> /dev/null
 if [ $? -eq 0 ];then
  #LOCALCOPY_ORBITFILE_NOHEADER=`basename $RA_ORBIT_FILE .org`_noheader.org
  LOCALCOPY_ORBITFILE_NOHEADER=`basename $RA_ORBIT_FILE .org`_NH.org
  echo "WARNING: the predicted orbit file $RA_ORBIT_FILE contains a header, will make a loacal copy without a header called $LOCALCOPY_ORBITFILE_NOHEADER" >> /dev/stderr
  tail -n +15 $RA_ORBIT_FILE > "$LOCALCOPY_ORBITFILE_NOHEADER"
  RA_ORBIT_FILE="$LOCALCOPY_ORBITFILE_NOHEADER"
 fi
 #
 if [ ! -f `basename "$RA_ORBIT_FILE"` ];then
  echo "Making a local copy of the orbit file" >> /dev/stderr
  cp -v "$RA_ORBIT_FILE" `basename "$RA_ORBIT_FILE"`
 fi
 #
 rm -f ra_orbit
 ln -s `basename "$RA_ORBIT_FILE"` ra_orbit
 # double-check if the orbit file is here
 if [ ! -s "$RA_ORBIT_FILE" ];then
  echo "ERROR: cannot find the predicted orbit file $RA_ORBIT_FILE" >> /dev/stderr
  exit 1
 fi
 cd $STARTDIR
 ### Done with the orbit thing ###
 ### NEW -- update menurat_defaults ###
 replace_orbit_filename_in_menurat_defaults `basename "$RA_ORBIT_FILE"`
 if [ $? -ne 0 ];then
  echo "ERROR: there seems to be a problem with the orbit file!" >> /dev/stderr
  exit 1
 fi
 ######################################
}


function email_report_to_kirx {
 if [ "$USER" != "kirx" ];then
  return 1
 fi
 HOST=`hostname`
 HOST="@$HOST"
 
 NAME="$USER$HOST"
 
 DATETIME=`LANG=C date --utc`
 
 SCRIPTNAME=`basename $0`
 
 MSG="The script $0 has finished on $DATETIME at $PWD"

 curl --silent 'http://scan.sai.msu.ru/vast/anyemailkirx.php' --data-urlencode "name=$NAME running $SCRIPTNAME" --data-urlencode "message=$MSG" --data-urlencode 'submit=submit' &>/dev/null

 return 0
}

#####################################################################################################

#############################################
### Before doing anything else, check if we need just to update orbit and exit
#############################################
if [ "$1" == "update" ];then
 export SESSION_KEY=$(set_session_key) # or SESSION_KEY=`set_session_key`
 set_orbit_symlink
 if [ $? -ne 0 ];then
  echo "ERROR: cannot update the orbit file!!!"
  exit 1
 fi
 # Update also the source catalog
 ./autosched.sh update
 exit 0
fi
#############################################


############### This pice is to compute visibility for ALL sources ###############
if [ -z $1 ];then
 # Make sure we start in the script directory
 cd $FAKERAT_PATH
 # If no parameters are specified - compute visibility for all sources in autosched.sh
 echo "Removing old computation results" >> /dev/stderr
 rm -f results.txt processing_complete.txt
 #rm -f source_visibility_prediction__*.txt
 PROCESSING_START=`date +%s`
 # Re-compile C programs
 gcc -O3 -o check_gb_ha check_gb_ha.c -lm
 gcc -O3 -o check_pu_altaz check_pu_altaz.c
 gcc -O3 -o check_yg_alt check_yg_alt.c
 gcc -O3 -o fort_75_to_baseline_and_pa fort_75_to_baseline_and_pa.c -lm
 #
 if [ ! -d distance_to_tracking_station_and_elevation_for_date ];then
  echo "ERROR: cannot find the subdirectory distance_to_tracking_station_and_elevation_for_date/" >> /dev/stderr
  exit 1
 fi
 cd distance_to_tracking_station_and_elevation_for_date
 # touch all files to prevent the clock skew thing
 for TOUCHFILE in * ;do
  touch $TOUCHFILE
 done
 make
 if [ $? -ne 0 ];then
  echo "ERROR compiling the C module in distance_to_tracking_station_and_elevation_for_date/" >> /dev/stderr
  cd $FAKERAT_PATH
  exit 1
 fi
 cd $FAKERAT_PATH
 # Update the source catalog
 if [ ! -x ./autosched.sh ];then
  echo "ERROR cannot find an executable file ./autosched.sh" >> /dev/stderr
  exit 1
 fi
 ./autosched.sh update
 
 # Set session key
 export SESSION_KEY=$(set_session_key) # or SESSION_KEY=`set_session_key`
 
 # Set the number of parallel execution threads
 NUMBER_OF_THREADS=$(set_number_of_threads)
 
# ### Get the latest orbit ###
 set_orbit_symlink
# STARTDIR=$PWD
# if [ ! -d orbit ];then
#  mkdir orbit
# fi
# cd orbit
# if [ "$RA_ORBIT_FILE" = "automatic" ];then
#  # automatically get the latest orbit file
#  RA_ORBIT_FILE=$(get_latest_orbit)
#  # Delete the oldest orbit file
#  OLDEST_ORBIT_FILE=`ls -t RA*.org | tail -n 1`
#  # make sure the oldest orbit file is not the same as the new one
#  if [ "$OLDEST_ORBIT_FILE" != "$RA_ORBIT_FILE" ];then
#   rm -f "$OLDEST_ORBIT_FILE"
#  else
#   echo "Not deleting the old orbit file - it's the same as the new one!" >> /dev/stderr
#  fi
# fi
# # 
# # Check if the orbit file is here
# if [ ! -s "$RA_ORBIT_FILE" ];then
#  echo "ERROR: cannot find the predicted orbit file $RA_ORBIT_FILE" >> /dev/stderr
#  exit 1
# else
#  echo "Using the predicted orbit file $RA_ORBIT_FILE" >> /dev/stderr
# fi
# # Check if we need to chop-off it's head
# head -n 10 $RA_ORBIT_FILE |grep "META_START" &> /dev/null
# if [ $? -eq 0 ];then
#  LOCALCOPY_ORBITFILE_NOHEADER=`basename $RA_ORBIT_FILE .org`_noheader.org
#  echo "WARNING: the predicted orbit file $RA_ORBIT_FILE contains a header, will make a loacal copy without a header called $LOCALCOPY_ORBITFILE_NOHEADER" >> /dev/stderr
#  tail -n +15 $RA_ORBIT_FILE > "$LOCALCOPY_ORBITFILE_NOHEADER"
#  RA_ORBIT_FILE="$LOCALCOPY_ORBITFILE_NOHEADER"
# fi
# rm -f ra_orbit
# ln -s $RA_ORBIT_FILE ra_orbit
# # double-check if the orbit file is here
# if [ ! -s "$RA_ORBIT_FILE" ];then
#  echo "ERROR: cannot find the predicted orbit file $RA_ORBIT_FILE" >> /dev/stderr
#  exit 1
# fi
# cd $STARTDIR
# ### Done with the orbit thing ###
 
 # Cycle through all sources listed inside autosched.sh
 COUNTER=0
 NPROC=0
 mkdir /tmp/autofake_"$SESSION_KEY"_results
 ###############################################
 # Check that sources.radioastron at least seems OK
 if [ ! -f sources.radioastron ];then
  echo "ERROR: no RadioAstron input catalog sources.radioastron" >> /dev/stderr
  exit 1
 fi
 if [ ! -s sources.radioastron ];then
  echo "ERROR: empty RadioAstron input catalog file sources.radioastron" >> /dev/stderr
  exit 1
 fi
 file sources.radioastron | grep Unicode >> /dev/stderr
 if [ $? -eq 0 ];then
  echo "ERROR: non-ASCII characters in RadioAstron input catalog file sources.radioastron" >> /dev/stderr
  echo "####### Trying to print the problematic characters #######" >> /dev/stderr
  # The following trick will not work without setting the language
  export LANG=C
  cat sources.radioastron | dos2unix | dos2unix | grep --color=always '[^ -~]\+'
  echo "##########################################################" >> /dev/stderr
  exit 1
 fi
 ###############################################
 #grep 'source=' sources.radioastron | grep -v '!    source=' | grep -v '!     source=' | grep -v '!   source=' | grep -v '!  source=' | grep -v '! source=' | grep -v '!source=' | grep -v 'calibrator' | grep -v "fake source representing RadioAstron" | awk '{print $1}' | while read TARGET ;do
 cat sources.radioastron | grep 'source=' | grep -v '\!' | grep -v 'calibrator' | grep -v "fake source representing RadioAstron" | awk '{print $1}' | while read TARGET ;do
  # Each time we are about to start a new thread, touch all the results files
  # to prevent evil tmpwatch from removing the older results
  touch /tmp/autofake_"$SESSION_KEY"_results
  for i in /tmp/autofake_"$SESSION_KEY"_results/* ;do
   if [ -f $i ];then
    touch $i
   fi
  done
  #
  CURRENT_RESULTS_FILE=/tmp/autofake_"$SESSION_KEY"_results/results_`echo "$COUNTER" | awk '{printf "%05d",$1}'`.txt
  COUNTER=$[$COUNTER+1]
  ############################
  TARGET=${TARGET/"source="/}
  TARGET=${TARGET//"'"/}
  TARGET=${TARGET//","/}
  TARGET=`echo "$TARGET" | awk '{print $1}'`
  # Go multithread
  $0 $TARGET > $CURRENT_RESULTS_FILE &
  PIDS="$PIDS $! "
  NPROC=$[$NPROC+1]
  while [ $NPROC -gt $[$NUMBER_OF_THREADS-1] ] ;do
   # check each running process
   #echo "$PIDS"
   for PID in $PIDS ;do
    ps -p $PID &> /dev/null
    if [ $? -ne 0 ];then
     PIDS=${PIDS/" $PID "/}
     NPROC=$[$NPROC-1]
    fi
   done
   sleep 0.01
  done
  # Now we are sure no more than NUMBER_OF_THREADS procssess are runnning
  echo "$PIDS" > /tmp/autofake_"$SESSION_KEY"_pids.tmp
 done
 
 # Wait for the remeining processes to finish
 for PID in `cat /tmp/autofake_"$SESSION_KEY"_pids.tmp` ;do
  while true ;do
   ps -p $PID &> /dev/null
   if [ $? -ne 0 ];then
    break
   fi
   sleep 0.01
  done
 done
 rm -f /tmp/autofake_"$SESSION_KEY"_pids.tmp

 # Combine individual source results into a single output file 
 for i in /tmp/autofake_"$SESSION_KEY"_results/results_*.txt ;do
  cat $i
 done > results.txt
 echo "Results are written to results.txt" >> /dev/stderr
 ## Generate a good output filename
 OUTFILENAME_ORBIT_START_DATE=`ls -l orbit/ra_orbit | awk '{print $2}' FS='->' | awk '{print $1}' | awk '{print $1}' FS='-'`
 # make sure we get a reasonable filename even if the orbit links to a completely outlandish filename
 OUTFILENAME_ORBIT_START_DATE=`basename $OUTFILENAME_ORBIT_START_DATE .txt`
 OUTFILENAME_ORBIT_START_DATE=`basename $OUTFILENAME_ORBIT_START_DATE .org`
 OUTFILENAME_ORBIT_START_DATE=`basename $OUTFILENAME_ORBIT_START_DATE .dat`
 # last-ditch
 if [ -z "$OUTFILENAME_ORBIT_START_DATE" ];then
  OUTFILENAME_ORBIT_START_DATE="UNKNOWNORBIT"
 fi
 #
 OUTFILENAME_DATES="$DATE_START $DATE_STOP"
 OUTFILENAME_DATES="${OUTFILENAME_DATES// /_}"
 OUTFILENAME_DATES="${OUTFILENAME_DATES//:/}"
 OUTFILENAME="source_visibility_prediction__"$OUTFILENAME_DATES"__"$OUTFILENAME_ORBIT_START_DATE".txt"
 cp results.txt $OUTFILENAME
 if [ $? -eq 0 ];then
  echo "Results are copied from results.txt to $OUTFILENAME" >> /dev/stderr
 else
  echo "Hmmm, that is funny...
ERROR copying results.txt to $OUTFILENAME" >> /dev/stderr
 fi
 ##
 if [ -d autofake_results_dir ];then
  rm -rf autofake_results_dir &>/dev/null
 fi
 cp -r /tmp/autofake_"$SESSION_KEY"_results/  autofake_results_dir
 echo "Details are saved to autofake_results_dir" >> /dev/stderr
 date > processing_complete.txt
 PROCESSING_STOP=`date +%s`
 COMPUTING_TIME=`echo "($PROCESSING_STOP-$PROCESSING_START)/86400" | bc -ql | awk '{printf "%.3f",$1}'`
 echo "Processing time: $COMPUTING_TIME days"
 # Report
 email_report_to_kirx
 # Clean-up
 rm -rf /tmp/autofake_"$SESSION_KEY"_results/
 #
 exit 0
fi

############### What follows is needed to compute visibility for a given source ###############


############ Change to a temporary directory ############
WORK_DIR=/tmp/autofake_"$$"_`tr -cd a-zA-Z0-9 < /dev/urandom | head -c 12 ; echo ""`
mkdir $WORK_DIR
cd $WORK_DIR
# symlink all necessary files
for i in ground  menu_colors  menurat_defaults  menurat_defaults.1 .orbitlines_for_fakerat_command_files  orbit  sources.cat  stations.cat autosched.sh stations.dat sun_from_horizons.dat check_gb_ha check_pu_altaz check_yg_alt fort_75_to_baseline_and_pa distance_to_tracking_station_and_elevation_for_date sources.radioastron ;do
 ln -s "$FAKERAT_PATH"/"$i"
done
############

# playing with file descriptors to speed up while loop
exec 3<&0

SOURCE=$1
#
#############################################
# DERIVED PARAMETERS
DOY_START=`date --utc -d "$DATE_START" +%j`
UNIXSEC_START=`date --utc -d "$DATE_START" +%s`
if [ $? -ne 0 ];then
 echo "ERROR computing UNIXSEC_START" >> /dev/stderr
 exit 1
fi
# 2011-07-18 02:31:00 = 1310956260
if [ $UNIXSEC_START -lt 1310956260 ];then
 echo "ERROR UNIXSEC_START is too much in the past" >> /dev/stderr
 exit 1
fi
DOY_STOP=`date --utc -d "$DATE_STOP" +%j`
UNIXSEC_STOP=`date --utc -d "$DATE_STOP" +%s`
if [ $? -ne 0 ];then
 echo "ERROR computing UNIXSEC_STOP" >> /dev/stderr
 exit 1
fi
if [ $UNIXSEC_START -ge $UNIXSEC_STOP ];then
 echo "ERROR: UNIXSEC_START cannot be larger or equal to UNIXSEC_STOP" >> /dev/stderr
 exit 1
fi
#############################################

# get source position and name
#RA=`grep "$SOURCE" sources.radioastron | grep "source=" | head -n1 | awk '{print $2}' FS='RA=' | awk '{print $1}'`
#DEC=`grep "$SOURCE" sources.radioastron | grep "source=" | head -n1 | awk '{print $2}' FS='DEC=' | awk '{print $1}'`
RA=`grep -v '\!' sources.radioastron | grep "$SOURCE" | grep "source=" | head -n1 | awk '{print $2}' FS='RA=' | awk '{print $1}'`
if [ "$RA" = "" ];then
 # Is this source now commented-out???
 RA=`cat sources.radioastron | grep "$SOURCE" | grep "source=" | head -n1 | awk '{print $2}' FS='RA=' | awk '{print $1}'`
fi
if [ "$RA" = "" ];then
 echo "ERROR getting RA for the source $SOURCE from sources.radioastron" >> /dev/stderr
 exit 1
fi 
DEC=`grep -v '\!' sources.radioastron | grep "$SOURCE" | grep "source=" | head -n1 | awk '{print $2}' FS='DEC=' | awk '{print $1}'`
if [ "$DEC" = "" ];then
 # Is this source now commented-out???
 DEC=`cat sources.radioastron | grep "$SOURCE" | grep "source=" | head -n1 | awk '{print $2}' FS='DEC=' | awk '{print $1}'`
fi
if [ "$DEC" = "" ];then
 echo "ERROR getting DEC for the source $SOURCE from sources.radioastron" >> /dev/stderr
 exit 1
fi 
SOURCEPOSITION_RADEC="$RA $DEC"
echo "Source $SOURCE position from sources.radioastron: $SOURCEPOSITION_RADEC" >> /dev/stderr
SOURCENAME=$SOURCE

## Save source declination in degrees for future tests
SOURCE_DECLINATION_IN_DEG=`skycoor -d -j $SOURCEPOSITION_RADEC J2000 | awk '{print $2}'`

## Convert J2000 to B1950
SOURCEPOSITION_RADEC_B1950=`skycoor -b $SOURCEPOSITION_RADEC J2000`
echo "Source position converted to B1950: $SOURCEPOSITION_RADEC_B1950" >> /dev/stderr
SOURCEPOSITION_RA=`echo $SOURCEPOSITION_RADEC_B1950 | awk '{print $1}'`
SOURCEPOSITION_DEC=`echo $SOURCEPOSITION_RADEC_B1950 | awk '{print $2}'`

UNIXSEC=`echo "$UNIXSEC_START-3600"|bc -q`

# Time loop (the main loop)
while [ $UNIXSEC -lt $UNIXSEC_STOP ];do

 #echo -n "Investigating $SOURCENAME  " >> /dev/stderr

 #UNIXSEC=`echo "$UNIXSEC+3600"|bc -q`
 # Replacing with native BASH hoping that it might be faster
 UNIXSEC=$[$UNIXSEC+3600]

 DATE_FROM_UNIXSEC_FOR_SUN=`date --utc -d "@$UNIXSEC" +"%Y-%m-%d %H:%M"`
 SUN_RADEC=`grep "$DATE_FROM_UNIXSEC_FOR_SUN" sun_from_horizons.dat | awk '{print $3":"$4":"$5" "$6":"$7":"$8}'`
 SUN_ANGLE_ARCSEC=`skycoor -r $SUN_RADEC $SOURCEPOSITION_RADEC`
 # TEST IF THE SOLAR ANGLE IS PROMISING
 #echo "$DATE_FROM_UNIXSEC_FOR_SUN $SUN_RADEC $SOURCEPOSITION_RADEC $SUN_ANGLE_DEG" >> /dev/stderr
 #TEST=`echo "$SUN_ANGLE_DEG>92.0"|bc -ql`
 #if [ $TEST -eq 0 ];then
 # continue
 #fi
 # it was 160
 #TEST=`echo "$SUN_ANGLE_DEG<150.0"|bc -ql`
 #if [ $TEST -eq 0 ];then
 # continue
 #fi
 
 # Optimized version
 if [ `echo "$SUN_ANGLE_ARCSEC>80.5*3600.0 && $SUN_ANGLE_ARCSEC<168.0*3600.0"|bc -ql` -eq 0 ];then
  # If the angle is VERY bad, skip 10 days
  UNIXSEC=$[$UNIXSEC+864000]
  continue
 fi
 if [ `echo "$SUN_ANGLE_ARCSEC>89.5*3600.0 && $SUN_ANGLE_ARCSEC<159.0*3600.0"|bc -ql` -eq 0 ];then
  # If the angle is bad, skip one day
  UNIXSEC=$[$UNIXSEC+86400]
  continue
 fi
 if [ `echo "$SUN_ANGLE_ARCSEC>90.5*3600.0 && $SUN_ANGLE_ARCSEC<158.0*3600.0"|bc -ql` -eq 0 ];then
  # If the angle is just ouside limits, don't skip, try next time interval
  continue
 fi
 # Conduct the following computation only if the solar angle is good
 SUN_ANGLE_DEG=`echo "$SUN_ANGLE_ARCSEC/3600" | bc -ql | awk '{printf "%5.1f",$1}'` 

 UNIXSEC_HOUR_STOP=`echo "$UNIXSEC+3600"|bc -q`

#
# Switching the custom module off as we want to test aslo YG and HH as tracking stations for the grav. exp.
# It seems safe to bet that one of the stations: PU GB YG HH will almost always see the satellite
# 
# # Test for elevation at the tracking stations with a custom NOVAS-based C module
# # so we don't need to run fakerat twice to figure out that the sattelite is below
# # horizon at both stations
# TEST=`distance_to_tracking_station_and_elevation_for_date/distance_to_ts_elevation orbit/ra_orbit $UNIXSEC $UNIXSEC_HOUR_STOP | tail -n1 | awk '{if ( $4 > 5 || $5 > 5 )print "1" ;else print "0"}'`
# if [ $? -ne 0 ];then
#  #continue
#  # The fast test code failed for whatever reason, assume the test is pased
#  # and perfrom the usual slow test with fakerat
#  TEST=1
# fi
# #echo "#$TEST# (0 - the sattelite is below horizon at the two tracking stations, 1 - it is up for at least one)" >> /dev/stderr
# if [ $TEST -ne 1 ];then
#  continue
# fi
 ###


 # move it here so we compute this only if SOLAR ANGLE is good
 YEAR=`date --utc -d "@$UNIXSEC" +%Y`
 DOY=`date --utc -d "@$UNIXSEC" +%j`
 HOUR_START=`date --utc -d "@$UNIXSEC" +%H | awk '{printf "%02d",$1}'`
 HOUR_STOP=`date --utc -d "@$UNIXSEC_HOUR_STOP" +%H |bc -q | awk '{printf "%02d",$1}'`

 UNIXSEC_VIRK_TECHNICAL_TIME_START=`echo "$UNIXSEC-15*60"|bc -q`
 YEAR_VIRK_TECHNICAL_TIME_START=`date --utc -d "@$UNIXSEC_VIRK_TECHNICAL_TIME_START" +%Y`
 DOY_VIRK_TECHNICAL_TIME_START=`date --utc -d "@$UNIXSEC_VIRK_TECHNICAL_TIME_START" +%j`
 HOUR_START_VIRK_TECHNICAL_TIME_START=`date --utc -d "@$UNIXSEC_VIRK_TECHNICAL_TIME_START" +%H | awk '{printf "%02d",$1}'`
 MINUTES_START_VIRK_TECHNICAL_TIME_START=`date --utc -d "@$UNIXSEC_VIRK_TECHNICAL_TIME_START" +%M | awk '{printf "%02d",$1}'`


 if [ ! -f .orbitlines_for_fakerat_command_files ];then
  echo "ERROR: cannot find .orbitlines_for_fakerat_command_files" >> /dev/stderr
  exit 1
 fi
 ORBITLINES_VAR=`cat .orbitlines_for_fakerat_command_files`

 rm -f def-azel-* .fakesat_plotazel.dat

 # Check the sattelite visibility for the TS
 echo "setenv PATH $FAKERAT_PATH:$FAKERATLIB_PATH:$PATH
fakerat <<EOD

NODATA
PLOTAZEL
TYPSAT = 2
TELEM      =
PUSCHINO
YEAR       = $YEAR_VIRK_TECHNICAL_TIME_START
DAY        =  $DOY_VIRK_TECHNICAL_TIME_START
START      = $HOUR_START_VIRK_TECHNICAL_TIME_START:$MINUTES_START_VIRK_TECHNICAL_TIME_START:00
STOP       = $HOUR_STOP:05:00
FREQ       = 5000
RA         = 00:00:00 !dummy variable
DEC        = 00:00:00 !dummy variable
INTEG      = 30
BNAME      = C
RA         = 00:00:00 !dummy variable
DEC        = 00:00:00 !dummy variable
$ORBITLINES_VAR
STATIONS   = RASTRON  , EFFELSBRG
LINEWIDTH  = 3
DOTSIZ     = 5
PLOT2F  /null
USEAEIOWM
AEIOWM(1)    171561556.00
AEIOWM(2) 0.9604
AEIOWM(3) 51.3815
AEIOWM(4) 331.000
AEIOWM(5) 292.000
AEIOWM(6) 0.010
YORB 2009
MORB  1
DORB   1
UTORB 0.00
/
EOD
exit
" > autofake-tmp-azel.csh
 nice csh autofake-tmp-azel.csh &> /dev/null

 #echo "################### DEBUG ###################" >>/dev/stderr
 #cat .fakesat_plotazel.dat autofake-tmp-azel.csh >>/dev/stderr
 #echo "#############################################" >>/dev/stderr
 #pwd
 #exit 1 # !!
 
 rm -f azel_problem.tmp
 #grep "    1  " .fakesat_plotazel.dat | while read ONE AZ X EL Y RANGE REST ;do
 # A faster version 
 #grep "    1  " .fakesat_plotazel.dat > /tmp/fast$$ 
 cat .fakesat_plotazel.dat | tail -n +6 | grep "    1  " > /tmp/fast$$
 #cat /tmp/fast$$
 # playing with file descriptors to speed up while loop
 exec 0< /tmp/fast$$
 while read ONE AZ X EL Y RANGE REST ;do
  ./check_pu_altaz $EL $AZ
   if [ $? -eq 1 ];then
    touch azel_problem.tmp
    break
   fi
 #
 # if [ `echo "$AZ<350.0 && $AZ>10.0 && $EL>15.0 && $EL<80.0"|bc -ql` -eq 1 ];then
 #  continue
 # fi
 # touch azel_problem.tmp
 #
 done # parsing azel file
 # playing with file descriptors to speed up while loop
 exec 0<&3
 # remove tmp file
 rm -f /tmp/fast$$ &

 # Reset the variable
 ACTIVE_TS=""
 if [ ! -f azel_problem.tmp ];then
  ACTIVE_TS="PU"
 else
  if [ $DEBUGLEVEL -gt 0 ];then
   echo "PU elvation check failed!" >> /dev/stderr
  fi
 fi

 rm -f def-azel-* .fakesat_plotazel.dat

 rm -f azel_problem.tmp
 echo "setenv PATH $FAKERAT_PATH:$FAKERATLIB_PATH:$PATH
fakerat <<EOD

NODATA
PLOTAZEL
TYPSAT = 2
TELEM      =
GBANK-5
YEAR       = $YEAR_VIRK_TECHNICAL_TIME_START
DAY        =  $DOY_VIRK_TECHNICAL_TIME_START
START      = $HOUR_START_VIRK_TECHNICAL_TIME_START:$MINUTES_START_VIRK_TECHNICAL_TIME_START:00
STOP       = $HOUR_STOP:05:00
FREQ       = 5000
RA         = 00:00:00 !dummy variable
DEC        = 00:00:00 !dummy variable
INTEG      = 30
BNAME      = C
RA         = 00:00:00 !dummy variable
DEC        = 00:00:00 !dummy variable
$ORBITLINES_VAR
STATIONS   = RASTRON  , EFFELSBRG
LINEWIDTH  = 3
DOTSIZ     = 5
PLOT2F  /null
USEAEIOWM
AEIOWM(1)    171561556.00
AEIOWM(2) 0.9604
AEIOWM(3) 51.3815
AEIOWM(4) 331.000
AEIOWM(5) 292.000
AEIOWM(6) 0.010
YORB 2009
MORB  1
DORB   1
UTORB 0.00
/
EOD
exit
" > autofake-tmp-azel.csh
 nice csh autofake-tmp-azel.csh &> /dev/null



 rm -f azel_problem.tmp
 #grep "    1  " .fakesat_plotazel.dat | while read ONE AZ X EL Y RANGE REST ;do
 #grep "    1  " .fakesat_plotazel.dat > /tmp/fast$$
 cat .fakesat_plotazel.dat | tail -n +6 | grep "    1  " > /tmp/fast$$
 exec 0< /tmp/fast$$
 while read ONE AZ X EL Y RANGE REST ;do
  ./check_gb_ha $EL $AZ
  if [ $? -eq 1 ];then
   touch azel_problem.tmp
   ## AAAAA
   #echo "./check_gb_ha $EL $AZ"
   #cp /tmp/fast$$ /tmp/PROBLEM.log
   #cp .fakesat_plotazel.dat /tmp/PROBLEM_in.log
   #exit 1
   ##
   break
  fi
 done # parsing azel file

 # playing with file descriptors to speed up while loop
 exec 0<&3
 # remove tmp file
 rm -f /tmp/fast$$ 

 if [ ! -f azel_problem.tmp ];then
  ACTIVE_TS="$ACTIVE_TS GB"
 else
  if [ $DEBUGLEVEL -gt 0 ];then
   echo "GB elvation check failed!" >> /dev/stderr
  fi
 fi

 if [ ! -z "$EXTRA_TRACKING_STATIONS" ];then
  if [ "$EXTRA_TRACKING_STATIONS" = "yes" ];then
   ############### Highly experimental - new TS for grav. exp. ###############
   rm -f def-azel-* .fakesat_plotazel.dat

   # Check the sattelite visibility for the TS
   echo "setenv PATH $FAKERAT_PATH:$FAKERATLIB_PATH:$PATH
fakerat <<EOD

NODATA
PLOTAZEL
TYPSAT = 2
TELEM      =
YARR
YEAR       = $YEAR
DAY        =  $DOY
START      = $HOUR_START:00:00
STOP       = $HOUR_STOP:00:00
FREQ       = 5000
RA         = 00:00:00 !dummy variable
DEC        = 00:00:00 !dummy variable
INTEG      = 30
BNAME      = C
RA         = 00:00:00 !dummy variable
DEC        = 00:00:00 !dummy variable
$ORBITLINES_VAR
STATIONS   = RASTRON  , EFFELSBRG
LINEWIDTH  = 3
DOTSIZ     = 5
PLOT2F  /null
USEAEIOWM
AEIOWM(1)    171561556.00
AEIOWM(2) 0.9604
AEIOWM(3) 51.3815
AEIOWM(4) 331.000
AEIOWM(5) 292.000
AEIOWM(6) 0.010
YORB 2009
MORB  1
DORB   1
UTORB 0.00
/
EOD
exit
" > autofake-tmp-azel.csh
   nice csh autofake-tmp-azel.csh &> /dev/null

   #echo "################### DEBUG ###################" >>/dev/stderr
   #cat .fakesat_plotazel.dat autofake-tmp-azel.csh >>/dev/stderr
   #echo "#############################################" >>/dev/stderr
   #pwd
   #exit 1 # !!
 
   rm -f azel_problem.tmp
   #grep "    1  " .fakesat_plotazel.dat | while read ONE AZ X EL Y RANGE REST ;do
   # A faster version 
   grep "    1  " .fakesat_plotazel.dat > /tmp/fast$$ 
   #cat /tmp/fast$$
   # playing with file descriptors to speed up while loop
   exec 0< /tmp/fast$$
   while read ONE AZ X EL Y RANGE REST ;do
    ./check_yg_alt $EL $AZ
    if [ $? -eq 1 ];then
     touch azel_problem.tmp
     break
    fi
   done # parsing azel file
   # playing with file descriptors to speed up while loop
   exec 0<&3
   # remove tmp file
   rm -f /tmp/fast$$ &

   if [ ! -f azel_problem.tmp ];then
    ACTIVE_TS="$ACTIVE_TS YG"
   fi

   rm -f def-azel-* .fakesat_plotazel.dat

   rm -f azel_problem.tmp
   echo "setenv PATH $FAKERAT_PATH:$FAKERATLIB_PATH:$PATH
fakerat <<EOD

NODATA
PLOTAZEL
TYPSAT = 2
TELEM      =
HARTS
YEAR       = $YEAR
DAY        =  $DOY
START      = $HOUR_START:00:00
STOP       = $HOUR_STOP:00:00
FREQ       = 5000
RA         = 00:00:00 !dummy variable
DEC        = 00:00:00 !dummy variable
INTEG      = 30
BNAME      = C
RA         = 00:00:00 !dummy variable
DEC        = 00:00:00 !dummy variable
$ORBITLINES_VAR
STATIONS   = RASTRON  , EFFELSBRG
LINEWIDTH  = 3
DOTSIZ     = 5
PLOT2F  /null
USEAEIOWM
AEIOWM(1)    171561556.00
AEIOWM(2) 0.9604
AEIOWM(3) 51.3815
AEIOWM(4) 331.000
AEIOWM(5) 292.000
AEIOWM(6) 0.010
YORB 2009
MORB  1
DORB   1
UTORB 0.00
/
EOD
exit
" > autofake-tmp-azel.csh
   nice csh autofake-tmp-azel.csh &> /dev/null

   rm -f azel_problem.tmp
   #grep "    1  " .fakesat_plotazel.dat | while read ONE AZ X EL Y RANGE REST ;do
   grep "    1  " .fakesat_plotazel.dat > /tmp/fast$$
   exec 0< /tmp/fast$$
   while read ONE AZ X EL Y RANGE REST ;do
    ./check_gb_ha $EL $AZ
    if [ $? -eq 1 ];then
     touch azel_problem.tmp
     break
    fi
   done # parsing azel file

   # playing with file descriptors to speed up while loop
   exec 0<&3
   # remove tmp file
   rm -f /tmp/fast$$ 

   if [ ! -f azel_problem.tmp ];then
    ACTIVE_TS="$ACTIVE_TS HH"
   fi

   ###########################################################################
 fi # if [ "$EXTRA_TRACKING_STATIONS" = "yes" ];then
fi # if [ ! -z "$EXTRA_TRACKING_STATIONS" ];then
  

 if [ "$ACTIVE_TS" == "" ];then
  continue
 fi

 # TS loop
 CONFIRMED_TS=""
 OK_TELESCOPE_SAVED=0
 OK_TELESCOPE=0
 #echo "ACTIVE_TS= $ACTIVE_TS" >> /dev/stderr
 for CURRENT_TS in $ACTIVE_TS ;do
  
  CURRENT_TS=${CURRENT_TS//PU/PUSCHINO}
  CURRENT_TS=${CURRENT_TS//GB/GBANK-5}
  #
  CURRENT_TS=${CURRENT_TS//YG/YARR}
  CURRENT_TS=${CURRENT_TS//HH/HARTS}
  #
 
  if [ $OK_TELESCOPE -eq 1 ];then
   OK_TELESCOPE_SAVED=1
  fi
  OK_TELESCOPE=0
  

  # Telescope loop
  for FAKERATTELESCOPE in EFFELSBR ATCA GBT KVNYS ;do
  #for FAKERATTELESCOPE in EFFELSBR EVPATORI GBT ATCA USUDA KVNYS HARTRAO ;do

   #echo "LOOPING $ACTIVE_TS $FAKERATTELESCOPE $OK_TELESCOPE_SAVED"


   # Skip EFFELSBR if hopeless
   if [ "$FAKERATTELESCOPE" == "EFFELSBR" ];then
    TEST=`echo "$SOURCE_DECLINATION_IN_DEG<-32.0"|bc -ql`
    if [ $TEST -eq 1 ];then
     continue  
    fi
   fi
 
   # Skip ATCA if hopeless
   if [ "$FAKERATTELESCOPE" == "ATCA" ];then
    #TEST=`echo "$SOURCE_DECLINATION_IN_DEG>47.0"|bc -ql`
    TEST=`echo "$SOURCE_DECLINATION_IN_DEG>40.0"|bc -ql`
    if [ $TEST -eq 1 ];then
     continue  
    fi
   fi
 
   # Skip GBT if hopeless
   if [ "$FAKERATTELESCOPE" == "GBT" ];then
    #TEST=`echo "$SOURCE_DECLINATION_IN_DEG<-45.0"|bc -ql`
    TEST=`echo "$SOURCE_DECLINATION_IN_DEG<-40.0"|bc -ql`
    if [ $TEST -eq 1 ];then
     continue  
    fi
   fi
 
   # Skip everythin else if hopeless OLD
   if [ "$FAKERATTELESCOPE" != "ATCA" ] && [ "$FAKERATTELESCOPE" != "HARTRAO" ] ;then
    TEST=`echo "$SOURCE_DECLINATION_IN_DEG<-45.0"|bc -ql`
    if [ $TEST -eq 1 ];then
     continue  
    fi
   fi
 
   # Skip ARECIBO if hopeless
   if [ "$FAKERATTELESCOPE" == "ARECIBO" ];then
    TEST=`echo "$SOURCE_DECLINATION_IN_DEG>38.0"|bc -ql`
    if [ $TEST -eq 1 ];then
     continue  
    fi
    TEST=`echo "$SOURCE_DECLINATION_IN_DEG<0.0"|bc -ql`
    if [ $TEST -eq 1 ];then
     continue  
    fi
   fi
 
   # Remove temporary files from a previous run 
   rm -f .fakesat_plotazel.dat fort.75 fort.50
   #
   rm -f def-uv-* def-azel-*
 
   # Launch the program
   echo "setenv PATH $FAKERAT_PATH:$FAKERATLIB_PATH:$PATH
fakerat <<EOD
UVPLOT
NODATA
TYPSAT = 2
CONSTRA
MODEL  model/none
SNRFLAG   -1.0
USEFWHM
LOGFWHM1     -1.30103004
LOGFWHM2       -1.30103004
LOGFWHM3    1.0
LOGS1        0.00000000
LOGS2        0.00000000
LOGS3    1.0
RATIO   1.0
PA        0.0
TELEM      =
$CURRENT_TS
YEAR       = $YEAR_VIRK_TECHNICAL_TIME_START
DAY        =  $DOY_VIRK_TECHNICAL_TIME_START
START      = $HOUR_START_VIRK_TECHNICAL_TIME_START:$MINUTES_START_VIRK_TECHNICAL_TIME_START:00
STOP       = $HOUR_STOP:05:00
FREQ       = 5000
BAND      = 16
INTEG      = 1000
BNAME      = C
SOURCE     = TEST
RA         = $SOURCEPOSITION_RA
DEC        = $SOURCEPOSITION_DEC
$ORBITLINES_VAR
STATIONS   = RASTRON  ,
$FAKERATTELESCOPE
LINEWIDTH  = 3
DOTSIZ     = 5
UMAX       =  5.00E9
VMAX       =  5.00E9
NGRDIL     = 5
NUP     = 2
PLOTF  /null
USEAEIOWM
AEIOWM(1)    171561556.00
AEIOWM(2) 0.9604
AEIOWM(3) 51.3815
AEIOWM(4) 331.000
AEIOWM(5) 292.000
AEIOWM(6) 0.010
YORB 2009
MORB  1
DORB   1
UTORB 0.00
/
EOD
exit

" > autofake-tmp.csh
   rm -f fort.75
   #
   rm -f def-uv-*
   nice csh autofake-tmp.csh &>/dev/null
   cp -- def-uv-*-d*-* fort.75
   # The following C insert will parse fort.75 and exit with 1 if something is wrong
   #echo "#################################################"
   #echo "$HOUR_START:00:00 $HOUR_STOP:00:00 $CURRENT_TS $FAKERATTELESCOPE"
   #echo "*************************************************"
   #cat fort.75
   #echo "#################################################"
   #exit 1
   ./fort_75_to_baseline_and_pa &>/dev/null
   if [ $? -ne 0 ];then
    OK_TELESCOPE=0
    continue
   else
    OK_TELESCOPE=1
    CONFIRMED_TS="$CONFIRMED_TS $CURRENT_TS"
    BASE=`./fort_75_to_baseline_and_pa`
    #echo "CONFIRMED $OK_TELESCOPE"
    break
   fi 
   # Nothing to do here: continue or break above
  done # Telescope loop

  if [ $OK_TELESCOPE -eq 1 ];then
   OK_TELESCOPE_SAVED=1
  fi
 
 done # TS loop


 # If not a single ground telescope was found
 if [ $OK_TELESCOPE_SAVED -eq 0 ];then
  continue
 fi

 CONFIRMED_TS=${CONFIRMED_TS/PUSCHINO/PU}
 CONFIRMED_TS=${CONFIRMED_TS/GBANK-5/GB}
 #
 CONFIRMED_TS=${CONFIRMED_TS/YARR/YG}
 CONFIRMED_TS=${CONFIRMED_TS/HARTS/HH}
 #
 # Remove blank space
 CONFIRMED_TS=${CONFIRMED_TS//" "/}
 # Set string length
 while [ ${#CONFIRMED_TS} -lt 8 ];do
  CONFIRMED_TS="$CONFIRMED_TS "
 done



 # Output the good visibility time
 DATE_FROM_UNIXSEC=`date --utc -d "@$UNIXSEC" +"%d.%m.%Y"`
 #echo -n "$UNIXSEC  $DATE_FROM_UNIXSEC $HOUR_START:00-$HOUR_STOP:00  $SOURCE  $SOURCENAME  $BASE"
 echo -n "$UNIXSEC  $DATE_FROM_UNIXSEC $HOUR_START:00-$HOUR_STOP:00  $SOURCENAME  $BASE"
 
 echo "$CONFIRMED_TS" | grep --quiet -e 'GB' -e 'PU'
 if [ $? -eq 0 ];then
  # We have a RadioAstron tracking station - let's get the detailed list of ground telescopes
  ALL_TELESCOPES=`nice ./autosched.sh $DATE_FROM_UNIXSEC $HOUR_START:00 $SOURCE 60`
  #
  GBT_DAY_OR_NIGHT=`distance_to_tracking_station_and_elevation_for_date/day_or_night_in_greenbank $DATE_FROM_UNIXSEC $HOUR_START:00`
  ALL_TELESCOPES=${ALL_TELESCOPES/Gb(/Gb($GBT_DAY_OR_NIGHT}
  #
 else
  # No RadioAstron tracking stations - only the ones good for grav. exp.
  ALL_TELESCOPES=" NO_TRACKING_STATIONS"
 fi

 echo "  $SUN_ANGLE_DEG $CONFIRMED_TS  $ALL_TELESCOPES"
 
done # Time loop

cd - &> /dev/null
rm -rf $WORK_DIR
